///////////////////////////////////////////////
//                                           //
// VRLab Mental Vision Module 8: Rendering   //
// Author: Achille Peternier                 //
// Copyright: EPFL-VRLab 2006 (C)            //
//                                           //
///////////////////////////////////////////////
#include "../data/common.h"
#pragma comment(lib, "opengl32.lib")



//////////////
// #DEFINES //
//////////////
#define MODULE_NAME "Rendering"



////////////
// GLOBAL //
////////////

int currentShading = 1;
float zoom = 250.0f;



//////////
// MAIN //
//////////

int main(int argc, char *argv[])
{ 
   /////////////////
   // Init MVisio //
   /////////////////
   INIT_MVISIO(MODULE_NAME, argc, argv);   
   
   // Common interface:
   USE_FONT();  
   USE_BACKGROUND(MODULE_SIZEX, MODULE_SIZEY);
   USE_QUITBUTTON(MODULE_SIZEX, MODULE_SIZEY);  
   USE_SELECTBOX();
   USE_CAMERA(0.0f, 32.0f, zoom, 0.0f, 0.0f, 0.0f); 
   USE_LIGHT(0.0f, 0.0f, 0.0f);    
   
   
   ////////////////////
   // Module interface:
   MVTEXTURE *back = new MVTEXTURE();
   back->load("blue.tga");
   
   MVWINDOW *winSettings = new MVWINDOW();
   winSettings->setFont(font);
   winSettings->setCaption("Shading model");
   winSettings->setSizes(253, 30);
   winSettings->setTexture(back);
   winSettings->setPosition((int) (MODULE_SIZEX/10), (int) (MODULE_SIZEY/10));
   
   MVBUTTON *flat = new MVBUTTON();
   flat->setFont(font);
   flat->setCaption("^x0015Flat");
   flat->setEventType(MV_BUTTON_SWITCH);
   flat->setSizes(60, 18);
   flat->setPosition(2, 5);
   flat->link(winSettings);
   
   MVBUTTON *gouraud = new MVBUTTON();
   gouraud->setFont(font);
   gouraud->setCaption("^x0004Gouraud");
   gouraud->setClickedStatus(true);
   gouraud->setEventType(MV_BUTTON_SWITCH);
   gouraud->setSizes(60, 18);
   gouraud->setPosition(64, 5);
   gouraud->link(winSettings);
   
   MVBUTTON *phong = new MVBUTTON();
   phong->setFont(font);
   phong->setCaption("^x0010Phong");
   phong->setEventType(MV_BUTTON_SWITCH);
   phong->setSizes(60, 18);
   phong->setPosition(126, 5);
   phong->link(winSettings);
   
   MVBUTTON *wireframe = new MVBUTTON();
   wireframe->setFont(font);
   wireframe->setCaption("Wireframe");
   wireframe->setEventType(MV_BUTTON_SWITCH);
   wireframe->setSizes(60, 18);
   wireframe->setPosition(188, 5);
   wireframe->link(winSettings);
   
   
   ///////////////////
   // Light interface:
   MVWINDOW *lightSettings = new MVWINDOW();
   lightSettings->setFont(font);
   lightSettings->setCaption("Light settings");
   lightSettings->setSizes(285, 222);
   lightSettings->setTexture(back);
   lightSettings->setPosition((int) (MODULE_SIZEX-lightSettings->getSizeX()*1.5), (int) (MODULE_SIZEY-lightSettings->getSizeY()*1.5)); 
     
   MVHSCROLLBAR *lAmbient[3];
   for (int c=0; c<3; c++)
   {
      lAmbient[c] = new MVHSCROLLBAR();
      lAmbient[c]->setSizeX(128);
      lAmbient[c]->setMinValue(0.0f);
      lAmbient[c]->setMaxValue(1.0f);
      lAmbient[c]->setValue(0.5);
      lAmbient[c]->setPosition(10, 18+c*8);
      lAmbient[c]->link(lightSettings);   
   }      
   lAmbient[0]->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   lAmbient[1]->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   lAmbient[2]->setColor(0.5f, 0.5f, 1.0f, 1.0f);   
   ((MVBOX *)lAmbient[0]->getChild(0))->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   ((MVBOX *)lAmbient[1]->getChild(0))->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   ((MVBOX *)lAmbient[2]->getChild(0))->setColor(0.5f, 0.5f, 1.0f, 1.0f);   
   
   MVHSCROLLBAR *lDiffuse[3];
   for (int c=0; c<3; c++)
   {
      lDiffuse[c] = new MVHSCROLLBAR();
      lDiffuse[c]->setSizeX(128);
      lDiffuse[c]->setMinValue(0.0f);
      lDiffuse[c]->setMaxValue(1.0f);
      lDiffuse[c]->setValue(0.5);
      lDiffuse[c]->setPosition(10, 68+c*8);
      lDiffuse[c]->link(lightSettings);   
   }      
   lDiffuse[0]->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   lDiffuse[1]->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   lDiffuse[2]->setColor(0.5f, 0.5f, 1.0f, 1.0f);   
   ((MVBOX *)lDiffuse[0]->getChild(0))->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   ((MVBOX *)lDiffuse[1]->getChild(0))->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   ((MVBOX *)lDiffuse[2]->getChild(0))->setColor(0.5f, 0.5f, 1.0f, 1.0f);      
  
   MVHSCROLLBAR *lSpecular[3];
   for (int c=0; c<3; c++)
   {
      lSpecular[c] = new MVHSCROLLBAR();
      lSpecular[c]->setSizeX(128);
      lSpecular[c]->setMinValue(0.0f);
      lSpecular[c]->setMaxValue(1.0f);
      lSpecular[c]->setValue(0.5);
      lSpecular[c]->setPosition(10, 118+c*8);
      lSpecular[c]->link(lightSettings);   
   }      
   lSpecular[0]->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   lSpecular[1]->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   lSpecular[2]->setColor(0.5f, 0.5f, 1.0f, 1.0f);   
   ((MVBOX *)lSpecular[0]->getChild(0))->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   ((MVBOX *)lSpecular[1]->getChild(0))->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   ((MVBOX *)lSpecular[2]->getChild(0))->setColor(0.5f, 0.5f, 1.0f, 1.0f); 
   
   // Global ambient:
   MVHSCROLLBAR *lGlobal[3];
   for (int c=0; c<3; c++)
   {
      lGlobal[c] = new MVHSCROLLBAR();
      lGlobal[c]->setSizeX(128);
      lGlobal[c]->setMinValue(0.0f);
      lGlobal[c]->setMaxValue(1.0f);
      lGlobal[c]->setValue(0.5);
      lGlobal[c]->setPosition(10, 188+c*8);
      lGlobal[c]->link(lightSettings);   
   }      
   lGlobal[0]->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   lGlobal[1]->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   lGlobal[2]->setColor(0.5f, 0.5f, 1.0f, 1.0f);   
   ((MVBOX *)lGlobal[0]->getChild(0))->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   ((MVBOX *)lGlobal[1]->getChild(0))->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   ((MVBOX *)lGlobal[2]->getChild(0))->setColor(0.5f, 0.5f, 1.0f, 1.0f);                  
   
   
   
   //////////////////////
   // Material interface:
   MVWINDOW *materialSettings = new MVWINDOW();
   materialSettings->setFont(font);
   materialSettings->setCaption("Material settings");
   materialSettings->setSizes(290, 246);
   materialSettings->setTexture(back);
   materialSettings->setPosition((int) (MODULE_SIZEX/10), MODULE_SIZEY-materialSettings->getSizeY()*1.5); 
   
   MVHSCROLLBAR *mTransparency;
   mTransparency = new MVHSCROLLBAR();   
   mTransparency->setSizeX(128);
   mTransparency->setMinValue(0.0f);
   mTransparency->setMaxValue(1.0f);
   mTransparency->setValue(1.0);
   mTransparency->setPosition(10, 232);
   mTransparency->link(materialSettings);   
   mTransparency->setColor(1.0f, 1.0f, 1.0f, 1.0f);
   ((MVBOX *)mTransparency->getChild(0))->setColor(1.0f, 1.0f, 1.0f, 1.0f);
   
   MVHSCROLLBAR *mAmbient[3];
   for (int c=0; c<3; c++)
   {
      mAmbient[c] = new MVHSCROLLBAR();
      mAmbient[c]->setSizeX(128);
      mAmbient[c]->setMinValue(0.0f);
      mAmbient[c]->setMaxValue(1.0f);
      mAmbient[c]->setValue(0.5);
      mAmbient[c]->setPosition(10, 50+c*8);
      mAmbient[c]->link(materialSettings);   
   }      
   mAmbient[0]->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   mAmbient[1]->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   mAmbient[2]->setColor(0.5f, 0.5f, 1.0f, 1.0f);   
   ((MVBOX *)mAmbient[0]->getChild(0))->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   ((MVBOX *)mAmbient[1]->getChild(0))->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   ((MVBOX *)mAmbient[2]->getChild(0))->setColor(0.5f, 0.5f, 1.0f, 1.0f);   
   
   MVHSCROLLBAR *mDiffuse[3];
   for (int c=0; c<3; c++)
   {
      mDiffuse[c] = new MVHSCROLLBAR();
      mDiffuse[c]->setSizeX(128);
      mDiffuse[c]->setMinValue(0.0f);
      mDiffuse[c]->setMaxValue(1.0f);
      mDiffuse[c]->setValue(0.5);
      mDiffuse[c]->setPosition(10, 100+c*8);
      mDiffuse[c]->link(materialSettings);   
   }      
   mDiffuse[0]->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   mDiffuse[1]->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   mDiffuse[2]->setColor(0.5f, 0.5f, 1.0f, 1.0f);   
   ((MVBOX *)mDiffuse[0]->getChild(0))->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   ((MVBOX *)mDiffuse[1]->getChild(0))->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   ((MVBOX *)mDiffuse[2]->getChild(0))->setColor(0.5f, 0.5f, 1.0f, 1.0f);      
  
   MVHSCROLLBAR *mSpecular[3];
   for (int c=0; c<3; c++)
   {
      mSpecular[c] = new MVHSCROLLBAR();
      mSpecular[c]->setSizeX(128);
      mSpecular[c]->setMinValue(0.0f);
      mSpecular[c]->setMaxValue(1.0f);
      mSpecular[c]->setValue(0.5);
      mSpecular[c]->setPosition(10, 150+c*8);
      mSpecular[c]->link(materialSettings);   
   }      
   mSpecular[0]->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   mSpecular[1]->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   mSpecular[2]->setColor(0.5f, 0.5f, 1.0f, 1.0f);   
   ((MVBOX *)mSpecular[0]->getChild(0))->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   ((MVBOX *)mSpecular[1]->getChild(0))->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   ((MVBOX *)mSpecular[2]->getChild(0))->setColor(0.5f, 0.5f, 1.0f, 1.0f);        
   
   MVHSCROLLBAR *mShininess = new MVHSCROLLBAR();
   mShininess->setSizeX(128);
   mShininess->setMinValue(0.0f);
   mShininess->setMaxValue(128.0f);
   mShininess->setValue(1.0f);
   mShininess->setPosition(10, 200);
   mShininess->setColor(0.7f, 0.7f, 0.7f, 1.0f);
   ((MVBOX *)mShininess->getChild(0))->setColor(0.7f, 0.7f, 0.7f, 1.0f);
   mShininess->link(materialSettings);
   
   
   /////////////////
   // Error message:
   bool displayError = !MVSETTINGS::getSDLShaders();
   if (displayError)
         phong->setDisplayStatus(false);
         
   MVWINDOW *errorBox = new MVWINDOW();
   errorBox->setColor(0.8f, 0.2f, 0.2f, 0.9f);
   errorBox->setFont(font);
   errorBox->setSizes(300, 170);
   errorBox->setPosition(MODULE_SIZEX/2-errorBox->getSizeX()/2, MODULE_SIZEY/2-errorBox->getSizeY()/2);
   errorBox->setCaption("Warning!");
   errorBox->write("Phong shading requires a modern graphic card supporting Shaders Model 2 or better.\n\n");
   errorBox->write("Either your card is not supporting shaders or you didn't activate them: to do that, ");
   errorBox->write("open the file 'config.txt' in the 'data' directory and set 'MVISIO_useShaders' to 1. ");
   errorBox->write("If this module fails to run after this modification, roll back this value to 0 or run this module on a more recent PC.");   
   errorBox->format();
   
   MVBUTTON *closeMe = new MVBUTTON();
   closeMe->setFont(font);
   closeMe->setColor(0.5f, 0.5f, 0.8f, 1.0f);
   closeMe->setSizes(42, 18);
   closeMe->setCaption("Close");
   closeMe->setPosition(errorBox->getSizeX()-closeMe->getSizeX()-4, errorBox->getSizeY()-closeMe->getSizeY()-3);
   closeMe->link(errorBox);
   
   // Scene:   
   MVNODE *scene = MVISIO::load("renderingScene.MVE"); 
   MVNODE *sceneFlat = MVISIO::load("renderingSceneFlat.MVE");
   MVMATERIAL *pulse = ((MVMESH *)selectBox->getChild(0))->getMaterial();
   pulse->setAmbient(1.0f, 1.0f, 1.0f, 1.0f);
   pulse->setEmission(1.0f, 1.0f, 1.0f, 1.0f);
   MVNODE *currentScene = scene;
   selectBox->link(light);     
   MVOPENGL::cullingOff();
   MVSETTINGS::forceShadersOff();           
  
  
///////////////
// Mainloop: //
///////////////

   int mouseX = 0, mouseY = 0, oldMouseX = 0, oldMouseY = 0;   
   bool mouseBL = false, mouseBR = false;
   bool oldMouseBL = false, oldMouseBR = false;
   bool done = false;
   float rotationX = 0.0f, rotationY = 0.0f;
   MVID currentSelected = 0;
   MVMESH *currentObject = NULL;
   MVMATERIAL *currentMaterial = NULL;
   
   while (!done)   
	{			
	      ////////////////////////
	      // Rendering background:	   
	   	MVISIO::clear(true, true, false);	   		   	
			MVISIO::begin2D();			   
			   PASS_BACKGROUND;					     			  
			MVISIO::end2D();		
			
			
			////////////////
			// 3D rendering:     			
			MVISIO::begin3D(camera);
			   light->pass();
			   currentScene->pass();
			MVISIO::end3D();	
			if (mouseBL && !oldMouseBL && !mouseBR)					
			      currentSelected = MVISIO::select(mouseX, mouseY);	         			      
			
			
			////////////////////////
			// Rendering foreground:
			MVISIO::begin2D();				   
			   // Title:
			   font->putString(10, 13, MODULE_NAME);	 
            
			   // Interface:
			   PASS_QUITBUTTON;	
			   winSettings->pass();	
			   lightSettings->pass();
			   materialSettings->pass();	 	
			   
			   // Error box:
			   if (displayError)
			         errorBox->pass();	  			  			   			
			MVISIO::end2D();
			
			
			////////////////
			// Check events:			
			SDL_Event _event;			
			//SDL_PumpEvents();						
			while (SDL_PollEvent(&_event)) 
			{
					// Quit by click on X:
					if (_event.type == SDL_QUIT) 
							done = true;	
					
					// Resize?
				   if (_event.type == SDL_VIDEORESIZE)
				         MVSETTINGS::resize(0, 0, _event.resize.w, _event.resize.h);							
               
               // Quit by pressing ESC:
					if (_event.type == SDL_KEYDOWN) 
					{
							// Quit by ESC:
							if (_event.key.keysym.sym == SDLK_ESCAPE) 
							       done = true;									       
                    			       													
			      }															
			}     
			
			// Update keyboard:    
			Uint8 *array;
			int arraySize;
			array = SDL_GetKeyState(&arraySize);
			if (array[SDLK_PAGEUP])    
			{
               zoom += 200.0f * MVISIO::getFPSfactor();  
               if (zoom > 500.0f)
			            zoom = 500.0f;			             
         }							       								                     
         else
               if (array[SDLK_PAGEDOWN])
               {
                     zoom -= 200.0f * MVISIO::getFPSfactor();
                     if (zoom < 100.0f)
			                  zoom = 100.0f;                                          
               }		
			
			// Update mouse:
			oldMouseBR = mouseBR;
			oldMouseBL = mouseBL;
			int buttonState = SDL_GetMouseState(&mouseX, &mouseY);		
         if (buttonState&SDL_BUTTON(1))
					mouseBL = true;					
			else 
					mouseBL = false;
         if (buttonState&SDL_BUTTON(2) || buttonState&SDL_BUTTON(3))
					mouseBR = true;					
			else 
					mouseBR = false;					
					
         float deltaX = mouseX - oldMouseX;
         float deltaY = mouseY - oldMouseY;		
         oldMouseX = mouseX; oldMouseY = mouseY;		
         
         // Check MVisio events:
			MVID id = 0;
			bool used =	MVELEMENT::manageGUIEvents(0, mouseX, mouseY, mouseBL, mouseBR, &id);  
			if (PRESSED_QUITBUTTON(id))
			      done = true;
           
         if (id == closeMe->getId())
               displayError = false;			      
         
         // Rotate scene:
         if (mouseBL && mouseBR && !used)
			{
			            zoom += deltaY*2.0f;
			            if (zoom > 500.0f) zoom = 500.0f;
			            if (zoom < 100.0f) zoom = 100.0f;			            
			}
			else
			      if (mouseBL && !used)
			      {
			            // Rotate model:
			            rotationX += deltaY/4.0f;
			            rotationY += deltaX/4.0f;
			            if (rotationX > 68.0f) rotationX = 68.0f;
			            if (rotationX < -68.0f) rotationX = -68.0f;	
			            currentScene->setRotation(rotationX, rotationY, 0.0f);		
		         }		
         
         if (currentSelected && mouseBL && !used)
         {
               MVNODE *_node = currentScene->findByIdInHierarchy(currentSelected);
               if (_node)
               {
                     if (_node != currentObject)
                     {
                           currentObject = (MVMESH *) _node;
                           currentMaterial = currentObject->getMaterial();
                           mAmbient[0]->setValue(currentMaterial->getAmbient().r);
                           mAmbient[1]->setValue(currentMaterial->getAmbient().g);
                           mAmbient[2]->setValue(currentMaterial->getAmbient().b);
                           mDiffuse[0]->setValue(currentMaterial->getDiffuse().r);
                           mDiffuse[1]->setValue(currentMaterial->getDiffuse().g);
                           mDiffuse[2]->setValue(currentMaterial->getDiffuse().b);
                           mSpecular[0]->setValue(currentMaterial->getSpecular().r);
                           mSpecular[1]->setValue(currentMaterial->getSpecular().g);
                           mSpecular[2]->setValue(currentMaterial->getSpecular().b);
                           mShininess->setValue(currentMaterial->getShininess());
                           mTransparency->setValue(currentMaterial->getTransparency());
                     }                           
               }
               else               
                     currentObject = NULL;
         }  
         if (currentMaterial) 
         {     
               currentMaterial->setAmbient(mAmbient[0]->getValue(), mAmbient[1]->getValue(), mAmbient[2]->getValue(), 1.0f);
               currentMaterial->setDiffuse(mDiffuse[0]->getValue(), mDiffuse[1]->getValue(), mDiffuse[2]->getValue(), 1.0f);
               currentMaterial->setSpecular(mSpecular[0]->getValue(), mSpecular[1]->getValue(), mSpecular[2]->getValue(), 1.0f);
               currentMaterial->setShininess(mShininess->getValue());
               currentMaterial->setTransparency(mTransparency->getValue());
         }		         
         
         // Changing rendering mode:
         if (id == flat->getId())
         {
               glShadeModel(GL_FLAT);                              
               MVISIO::wireframeOff();
               MVSETTINGS::forceShadersOff();
               flat->setClickedStatus(true);
               gouraud->setClickedStatus(false);
               phong->setClickedStatus(false);
               wireframe->setClickedStatus(false);               
               currentShading = 0;               
         }		
         if (id == gouraud->getId())
         {
               glShadeModel(GL_SMOOTH);  
               MVISIO::wireframeOff();    
               MVSETTINGS::forceShadersOff();         
               flat->setClickedStatus(false);
               gouraud->setClickedStatus(true);
               phong->setClickedStatus(false);
               wireframe->setClickedStatus(false);               
               currentShading = 1;               
         }	
         if (id == phong->getId())
         {
               glShadeModel(GL_SMOOTH);          
               MVISIO::wireframeOff(); 
               MVSETTINGS::forceShadersOn();    
               flat->setClickedStatus(false);
               gouraud->setClickedStatus(false);
               phong->setClickedStatus(true);
               wireframe->setClickedStatus(false);               
               currentShading = 2;                              
         }				  	               
         if (id == wireframe->getId())
         {
               glShadeModel(GL_SMOOTH);          
               MVISIO::wireframeOn(); 
               MVSETTINGS::forceShadersOff();    
               flat->setClickedStatus(false);
               gouraud->setClickedStatus(false);
               phong->setClickedStatus(false);
               wireframe->setClickedStatus(true);               
               currentShading = 3;                              
         }		      
         
         // Update:         
         light->setAmbient(lAmbient[0]->getValue(), lAmbient[1]->getValue(), lAmbient[2]->getValue(), 1.0f);
         light->setDiffuse(lDiffuse[0]->getValue(), lDiffuse[1]->getValue(), lDiffuse[2]->getValue(), 1.0f);
         light->setSpecular(lSpecular[0]->getValue(), lSpecular[1]->getValue(), lSpecular[2]->getValue(), 1.0f); 
         MVOPENGL::setGlobalAmbientLight(lGlobal[0]->getValue(), lGlobal[1]->getValue(), lGlobal[2]->getValue(), 1.0f);  
         
         camera->setPosition(0.0f, 32.0f, zoom);          

         // Display text on material:         
         materialSettings->clear();
         if (currentObject)         
               materialSettings->write("Object selected: %s\n", currentObject->getName());                   
         else
               materialSettings->write("Object selected: [none]\n"); 
         materialSettings->write("\nAmbient:\n"); 
         materialSettings->write("^y0005                        RGB(^c8008%d, ^c0808%d, ^c4488%d^c8888)", (int) (mAmbient[0]->getValue()*255.0f), (int) (mAmbient[1]->getValue()*255.0f), (int) (mAmbient[2]->getValue()*255.0f));                            
         materialSettings->write("\n^y0013Diffuse:\n");                             
         materialSettings->write("^y0005                        RGB(^c8008%d, ^c0808%d, ^c4488%d^c8888)", (int) (mDiffuse[0]->getValue()*255.0f), (int) (mDiffuse[1]->getValue()*255.0f), (int) (mDiffuse[2]->getValue()*255.0f));                            
         materialSettings->write("\n^y0013Specular:\n");                             
         materialSettings->write("^y0006                        RGB(^c8008%d, ^c0808%d, ^c4488%d^c8888)", (int) (mSpecular[0]->getValue()*255.0f), (int) (mSpecular[1]->getValue()*255.0f), (int) (mSpecular[2]->getValue()*255.0f));                            
         materialSettings->write("\n^y0012Shininess:^y0011");                             
         materialSettings->write("               ^x0004value: %d", (int) (mShininess->getValue()));     
         materialSettings->write("\n^y0005Transparency:^y0011");
         materialSettings->write("            ^x0001value: %.2f", mTransparency->getValue());                       
         materialSettings->format();
         
         // Display text on light:
         lightSettings->clear();
         lightSettings->write("Ambient:\n");
         lightSettings->write("^y0005                        RGB(^c8008%d, ^c0808%d, ^c4488%d^c8888)", (int) (lAmbient[0]->getValue()*255.0f), (int) (lAmbient[1]->getValue()*255.0f), (int) (lAmbient[2]->getValue()*255.0f));                            
         lightSettings->write("\n^y0013Diffuse:\n");
         lightSettings->write("^y0005                        RGB(^c8008%d, ^c0808%d, ^c4488%d^c8888)", (int) (lDiffuse[0]->getValue()*255.0f), (int) (lDiffuse[1]->getValue()*255.0f), (int) (lDiffuse[2]->getValue()*255.0f));                            
         lightSettings->write("\n^y0013Specular:\n");
         lightSettings->write("^y0005                        RGB(^c8008%d, ^c0808%d, ^c4488%d^c8888)\n", (int) (lSpecular[0]->getValue()*255.0f), (int) (lSpecular[1]->getValue()*255.0f), (int) (lSpecular[2]->getValue()*255.0f));                            
         lightSettings->write(" ----------------------------------\nGlobal ambient:\n");
         lightSettings->write("^y0005                        RGB(^c8008%d, ^c0808%d, ^c4488%d^c8888)\n", (int) (lGlobal[0]->getValue()*255.0f), (int) (lGlobal[1]->getValue()*255.0f), (int) (lGlobal[2]->getValue()*255.0f));                            
         lightSettings->format();
         
         for (int c=0; c<currentScene->getNumberOfChildren(); c++)
         {
               float speed = MVISIO::getFPSfactor()*10.0f;
               currentScene->getChild(c)->rotate(c*speed, (2-c)*speed, (4-c)*speed);
			}
			
			// Display:		
			Sleep(10); // Don't kill mthreading ;)
			MVISIO::swap();					
 	}      
   
   // Done:
   MVISIO::free();
   return 0;
}